//------------------------------------------------------------------------------
// TscExcelExport : Export Delphi Dataset to MS Excel
// February - July 2000
// Version 1.4
//
// Stefan Cruysberghs
// stefancr@yucom.be (= stefancr@mail.dma.be)
// http://bewoner.antwerpen.be/stefancr (= http://www.bewoner.dma.be/stefancr)
//------------------------------------------------------------------------------
// This component is free of charge.
// The author doesn't give a warranty for error free running
// of this component and he doesn't give any support.
// Suggestions and bugs can be send by email.
//------------------------------------------------------------------------------
// Version 1.2
//  - Improved connection to Excel
// Version 1.3
//  - Added Orientation of titles
//  - Add StyleColumnWidth
// Version 1.4
//  - Improved GetColumnCharacters
//  - Added Border properties and background colors for Titles, Data and Summary
//  - Added Summary properties (SummarySelection, SummaryFields, SummaryCalculation)
// Future
//  - Remove warning 'constant expression violates subrange bounds' 
//------------------------------------------------------------------------------

{
  Example 1 : easiest way to use

  scExcelExport1.Dataset:=Table1;
  scExcelExport1.ExportDataset;
  scExcelExport1.Disconnect;
}

{
  Example 2

  scExcelExport1.WorksheetName := 'MyDataset';
  scExcelExport1.Dataset:=Table1;
  scExcelExport1.StyleColumnWidth:=cwOwnerWidth;
  scExcelExport1.ColumnWidth := 20;
  scExcelExport1.FontTitles := LabelTitle.Font;
  scExcelExport1.BorderTitles.BackColor := clYellow;
  scExcelExport1.BorderTitles.BorderColor := clRed;
  scExcelExport1.BorderTitles.LineStyle := blDoubleLine;
  scExcelExport1.OrientationTitles := 45;
  scExcelExport1.FontData := LabelData.Font;
  scExcelExport1.SummarySelection := ssValues;
  scExcelExport1.SummaryCalculation := scMAX;
  scExcelExport1.ExcelVisible:=False;
  scExcelExport1.ExportDataset;
  scExcelExport1.SaveAs('c:\test.xls',ffXLS);
  scExcelExport1.Disconnect;
}

unit scExcelExport;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, DB,
  StdCtrls, OleServer, Excel97;

  // When you have installed Service Pack 1 for Delphi 5
  // you can use the unit Excel2000
  // Otherwise use unit Excel97
  // (When using Excel2000 I got errors with package dclaxserver50)

type
  TFileFormat = (ffXLS, ffHTM, ffCSV);
  TStyleColumnWidth = (cwDefault, cwOwnerWidth, cwAutoFit);
  TBorderWeight = (bwThin, bwMedium, bwThick);
  TBorderLineStyle = (blNone, blDash, blLine, blDoubleLine);
  TSummarySelection = (ssNone, ssValues, ssGiven);
  TSummaryCalculation = (scSUM, scMIN, scMAX);

  TCellBorder = class(TPersistent)
  private
    FBackColor : TColor;
    FBorderColor : TColor;
    FBorderWeight : TBorderWeight;
    FBorderLineStyle : TBorderLineStyle;
  published
    property BackColor : TColor read FBackColor write FBackColor default clWhite;
    property BorderColor : TColor read FBorderColor write FBorderColor default clBlack;
    property Weight : TBorderWeight read FBorderWeight write FBorderWeight default bwMedium;
    property LineStyle : TBorderLineStyle read FBorderLineStyle write FBorderLineStyle default blNone;
  end;

  TscExcelExport = class(TComponent)
  private
    FDataset : TDataset;

    FExcelApplication : TExcelApplication;
    FExcelWorkbook : TExcelWorkbook;
    FExcelWorksheet : TExcelWorksheet;

    FFieldNames : TStrings;

    FBlnExcelVisible : Boolean;
    FStrWorksheetName : String;
    FIntColumnWidth : Integer;
    FStyleColumnWidth : TStyleColumnWidth;

    FFontTitles : TFont;
    FBorderTitles : TCellBorder;
    FFontOrientationTitles : Integer;

    FFontData : TFont;
    FBorderData : TCellBorder;

    FFontSummary : TFont;
    FBorderSummary : TCellBorder;
    FSummarySelection : TSummarySelection;
    FSummaryFields : TStrings;
    FSummaryCalculation : TSummaryCalculation;

    LCID : Integer;
  protected
    procedure SetFontTitles(Value : TFont);
    procedure SetFontData(Value : TFont);
    procedure SetFontSummary(Value : TFont);
    procedure SetSummaryFields(Value : TStrings);

    procedure SetColumnWidth;
    function GetColumnCharacters(IntNumber : Integer) : String;
    procedure SetFontAndBorderRange(DelphiFont : TFont; Border : TCellBorder; StrBeginCell, StrEndCell : String);

    function CanConvertFieldToCell(Field : TField) : Boolean;
    function IsValueField(Field : TField) : Boolean;

    procedure ExportTitles;
    procedure ExportFieldData;
    procedure ExportSummary;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;

    procedure Disconnect;
    procedure ExportDataset;
    procedure SaveAs(const StrFileName : String; const FileFormat : TFileFormat);
    procedure PrintPreview(const BlnPrintGridLines : Boolean);
  published
    property ExcelVisible : Boolean read FBlnExcelVisible write FBlnExcelVisible default True;
    property StyleColumnWidth : TStyleColumnWidth read FStyleColumnWidth write FStyleColumnWidth;
    property ColumnWidth : Integer read FIntColumnWidth write FIntColumnWidth;
    property WorksheetName : String read FStrWorksheetName write FStrWorksheetName;
    property Dataset : TDataset read FDataset write FDataset;

    property FontTitles : TFont read FFontTitles write SetFontTitles;
    property BorderTitles : TCellBorder read FBorderTitles write FBorderTitles;
    property OrientationTitles : Integer read FFontOrientationTitles write FFontOrientationTitles;

    property FontData : TFont read FFontData write SetFontData;
    property BorderData : TCellBorder read FBorderData write FBorderData;

    property FontSummary : TFont read FFontSummary write SetFontSummary;
    property BorderSummary : TCellBorder read FBorderSummary write FBorderSummary;
    property SummaryFields : TStrings read FSummaryFields write SetSummaryFields;
    property SummarySelection : TSummarySelection read FSummarySelection write FSummarySelection;
    property SummaryCalculation : TSummaryCalculation read FSummaryCalculation write FSummaryCalculation;
  end;

procedure Register;

implementation

uses ComObj, ActiveX;

procedure Register;
begin
  RegisterComponents('SC', [TscExcelExport]);
end;

{ $R scExcelExport.dcr }

//------------------------------------------------------------------------------
constructor TscExcelExport.Create(Owner: TComponent);
begin
  inherited;
  FFontTitles := TFont.Create;
  FFontData := TFont.Create;
  FFontSummary := TFont.Create;

  FBorderTitles := TCellBorder.Create;
  FBorderData := TCellBorder.Create;
  FBorderSummary := TCellBorder.Create;

  FBorderTitles.FBackColor := clWhite;
  FBorderTitles.FBorderColor := clBlack;
  FBorderTitles.FBorderWeight := bwMedium;
  FBorderTitles.FBorderLineStyle := blNone;

  FBorderData.FBackColor := clWhite;
  FBorderData.FBorderColor := clBlack;
  FBorderData.FBorderWeight := bwMedium;
  FBorderData.FBorderLineStyle := blNone;

  FBlnExcelVisible := True;
  FStrWorksheetName := '';
  FStyleColumnWidth := cwDefault;
  FIntColumnWidth := 0;
  FFontOrientationTitles := 0;

  FExcelApplication := TExcelApplication.Create(Self);
  FExcelWorkbook := TExcelWorkbook.Create(Self);
  FExcelWorksheet := TExcelWorksheet.Create(Self);

  FFieldNames:=TStringList.Create;
  FSummaryFields:=TStringList.Create;
end;

//------------------------------------------------------------------------------
destructor TscExcelExport.Destroy;
begin
  FFontTitles.Free;
  FFontData.Free;
  FFontSummary.Free;

  FBorderTitles.Free;
  FBorderData.Free;
  FBorderSummary.Free;

  FExcelWorksheet.Free;
  FExcelWorkbook.Free;
  FExcelApplication.Free;

  FFieldNames.Free;
  FSummaryFields.Free;
  inherited;
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.SetFontTitles(Value : TFont);
begin
  FFontTitles.Assign(Value);
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.SetFontData(Value : TFont);
begin
  FFontData.Assign(Value);
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.SetFontSummary(Value : TFont);
begin
  FFontSummary.Assign(Value);
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.SetSummaryFields(Value : TStrings);
begin
  FSummaryFields.Assign(Value);
  FSummaryFields.Text := UpperCase(FSummaryFields.Text);
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.SetFontAndBorderRange(DelphiFont : TFont; Border : TCellBorder; StrBeginCell, StrEndCell : String);
begin
  // Convert Delphi font to the Excel font
  with FExcelWorksheet.Range[StrBeginCell, StrEndCell].Font do
  begin
    Name := DelphiFont.Name;
    Size := DelphiFont.Size;
    Color := DelphiFont.Color;
    Bold :=  fsBold in DelphiFont.Style;
    Italic := fsItalic in DelphiFont.Style;
    Underline := fsUnderline in DelphiFont.Style;
  end;

  // Titles
  if StrBeginCell = 'A1' then
    FExcelWorksheet.Range[StrBeginCell, StrEndCell].Orientation:=FFontOrientationTitles;

  if Border.FBackColor <> clWhite then
    FExcelWorksheet.Range[StrBeginCell, StrEndCell].Interior.Color := Border.FBackColor;

  if Border.LineStyle <> blNone then
  begin
    with FExcelWorksheet.Range[StrBeginCell, StrEndCell].Borders do
    begin
      Color := Border.BorderColor;
      case Border.FBorderWeight of
        bwThin : Weight := xlThin;
        bwMedium : Weight := xlMedium;
        bwThick : Weight := xlThick;
      end;

      case Border.LineStyle of
        blDash : LineStyle := xlDash;
        blLine : LineStyle := xlLine;
        blDoubleLine : LineStyle := xlDouble;
      end;
    end;
  end;
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.SetColumnWidth;
begin
  if FStyleColumnWidth = cwOwnerWidth then
    FExcelWorksheet.Range['A1',GetColumnCharacters(FFieldNames.Count)+'1'].ColumnWidth:=FIntColumnWidth;
  if FStyleColumnWidth = cwAutoFit then
    FExcelWorksheet.Range['A1',GetColumnCharacters(FFieldNames.Count)+'1'].EntireColumn.Autofit;
end;

//------------------------------------------------------------------------------
function TscExcelExport.CanConvertFieldToCell(Field : TField) : Boolean;
begin
  if (Field.DataType = ftString) or
    (Field.DataType = ftSmallint) or
    (Field.DataType = ftInteger) or
    (Field.DataType = ftWord) or
    (Field.DataType = ftBoolean) or
    (Field.DataType = ftFloat) or
    (Field.DataType = ftCurrency) or
    (Field.DataType = ftBCD) or
    (Field.DataType = ftDate) or
    (Field.DataType = ftTime) or
    (Field.DataType = ftDateTime) then
  begin
    Result:=True;
  end
  else
    Result:=False;
end;

//------------------------------------------------------------------------------
function TscExcelExport.IsValueField(Field : TField) : Boolean;
begin
  if (Field.DataType = ftSmallint) or
    (Field.DataType = ftInteger) or
    (Field.DataType = ftWord) or
    (Field.DataType = ftFloat) or
    (Field.DataType = ftCurrency) then
  begin
    Result:=True;
  end
  else
    Result:=False;
end;


//------------------------------------------------------------------------------
// Get Column-character for giving index
//------------------------------------------------------------------------------
function TscExcelExport.GetColumnCharacters(IntNumber : Integer) : String;
begin
  if IntNumber < 1 then
    Result:='A'
  else
  begin
    if IntNumber > 702 then
      Result:='ZZ'
    else
    begin
      if IntNumber > 26 then begin
       if (IntNumber mod 26)=0 then
        Result:=Chr(64 + (IntNumber div 26)-1)
       else
        Result:=Chr(64 + (IntNumber div 26));
         if (IntNumber mod 26)=0 then
          result:=result+chr(64+26)
         else
          result:=Result+Chr(64 + (IntNumber mod 26));
      end
      else
        Result:=Chr(64 + IntNumber);
    end;
  end;
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.Disconnect;
begin
  if not FExcelApplication.Visible[LCID] then
  begin
    FExcelApplication.DisplayAlerts[LCID] := False;
    FExcelApplication.Quit;
  end;
  FExcelWorksheet.Disconnect;
  FExcelWorkbook.Disconnect;
  FExcelApplication.Disconnect;
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.ExportDataset;
begin
  Screen.Cursor := crHourGlass;
  try
    with FDataset do
      if not Active then
        Exit;

    LCID := GetUserDefaultLCID;

    // Try to connect to Excel and create new Worksheet
    try
      FExcelApplication.ConnectKind := ckNewInstance;
      FExcelApplication.Connect;
      FExcelWorkbook.ConnectTo(FExcelApplication.Workbooks.Add(TOleEnum(xlWBATWorksheet), LCID));
      FExcelWorksheet.ConnectTo(FExcelWorkbook.Worksheets[1] as _Worksheet);
    except
      Exit;
    end;

    FExcelApplication.ScreenUpdating[LCID] := False;

    // If property worksheetname is not filled, worksheet will have name of dataset
    if FStrWorksheetName <> '' then
      FExcelWorksheet.Name := FStrWorksheetName
    else
      FExcelWorksheet.Name := FDataset.Name;

    // Export fieldnames and data and calculate summary
    ExportTitles;
    ExportFieldData;
    if FSummarySelection <> ssNone then
      ExportSummary;

    // Set fonts of titles, data and summary
    SetFontAndBorderRange(FFontTitles, FBorderTitles, 'A1', GetColumnCharacters(FFieldNames.Count)+'1');
    SetFontAndBorderRange(FFontData, FBorderData, 'A2', GetColumnCharacters(FFieldNames.Count)+IntToStr(FDataset.RecordCount+1));
    if FSummarySelection <> ssNone then
      SetFontAndBorderRange(FFontSummary, FBorderSummary, 'A'+IntToStr(FDataset.RecordCount+2), GetColumnCharacters(FFieldNames.Count)+IntToStr(FDataset.RecordCount+2));

    // Set width of columns
    SetColumnWidth;

    FExcelApplication.ScreenUpdating[LCID] := FBlnExcelVisible;
    FExcelApplication.Visible[LCID]:=FBlnExcelVisible;
  finally
    Screen.Cursor := crDefault;
  end;
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.ExportTitles;
var
  IntRow : Integer;
  IntColumn : Integer;
  StrCell : String;
begin
  IntRow:=1;
  with FDataset do
  begin
    FFieldNames.Clear;
    for IntColumn := 1 to Fields.Count do
    begin
      // Only export fields which are writable in an Excel cell
      // Add these fields to a list, so this list can be used
      // when exporting data
      if CanConvertFieldToCell(Fields[IntColumn - 1]) then
      begin
        FFieldNames.Add(Fields[IntColumn-1].FieldName);
        StrCell:=GetColumnCharacters(IntColumn)+IntToStr(IntRow);

        // Use DisplayName of column if this is filled in, otherwise use FieldName
        if Fields[IntColumn-1].DisplayName <> '' then
          FExcelWorksheet.Range[StrCell,StrCell].Value := Fields[IntColumn-1].DisplayName
        else
          FExcelWorksheet.Range[StrCell,StrCell].Value := Fields[IntColumn-1].FieldName;
      end;
    end;
  end;
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.ExportFieldData;
var
  IntRow : Integer;
  IntColumn : Integer;
  StrCell : String;
  PtBookmark : TBookmark;
begin
  IntRow:=2;
  with FDataset do
  begin
    DisableControls;
    PtBookmark := GetBookmark;
    try
      First;
      while not EOF do
      begin
        for IntColumn := 1 to FFieldNames.Count do
        begin
          if FieldByName(FFieldNames[IntColumn-1]).AsString <> '' then
          begin
            StrCell:=GetColumnCharacters(IntColumn)+IntToStr(IntRow);
            FExcelWorksheet.Range[StrCell,StrCell].Value := FieldByName(FFieldNames[IntColumn-1]).Value;
          end;
        end;
        Next;
        Inc(IntRow);
      end;
    finally
      GotoBookmark(PtBookmark);
      FreeBookmark(PtBookmark);
      EnableControls;
    end;
  end;
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.ExportSummary;
var
  IntRow : Integer;
  IntColumn : Integer;
  StrCell : String;
  StrCalc : String;
begin
  with FDataset do
  begin
    IntRow:=RecordCount+2;
    for IntColumn := 1 to FFieldNames.Count do
    begin
      if ((FSummarySelection = ssValues) and (IsValueField(FieldByName(FFieldNames[IntColumn-1])))) or
         ((FSummarySelection = ssGiven) and (FSummaryFields.IndexOf(UpperCase(FFieldNames[IntColumn-1])) >= 0)) then
      begin
        StrCell:=GetColumnCharacters(IntColumn)+IntToStr(IntRow);
        case FSummaryCalculation of
          scSUM : StrCalc := 'SUM';
          scMIN : StrCalc := 'MIN';
          scMAX : StrCalc := 'MAX';
        end;

        FExcelWorksheet.Range[StrCell,StrCell].Value :=
          '='+StrCalc+'('+GetColumnCharacters(IntColumn)+'2:'+GetColumnCharacters(IntColumn)+IntToStr(IntRow-1)+')';
      end;
    end;
  end;
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.SaveAs(const StrFileName : String; const FileFormat : TFileFormat);
begin
  FExcelApplication.DisplayAlerts[LCID] := False;
  // Export data to a file
  case FileFormat of
    ffXLS : FExcelWorksheet.SaveAs(StrFileName,xlNormal);
    ffCSV : FExcelWorksheet.SaveAs(StrFileName,xlCSV);
    //ffHTM : FExcelWorksheet.SaveAs(StrFileName,xlHtml);  // Only works with Excel2000
  end;
end;

//------------------------------------------------------------------------------
procedure TscExcelExport.PrintPreview(const BlnPrintGridLines : Boolean);
begin
  // Show PrintPreview of Excel
  FExcelWorksheet.PageSetup.PrintGridlines:=BlnPrintGridLines;
  FExcelWorksheet.PageSetup.CenterHeader:=FExcelWorksheet.Name;
  FExcelApplication.ScreenUpdating[LCID] := True;
  FExcelApplication.Visible[LCID]:=True;
  FExcelWorksheet.PrintPreview;
end;

end.


